// dibdoc.cpp : implementation of the CDibDoc class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1997 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "diblook.h"
#include <limits.h>

#include "dibdoc.h"

#include "jpegdib.h"
#include "docrop.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDibDoc

IMPLEMENT_DYNCREATE(CDibDoc, CDocument)

BEGIN_MESSAGE_MAP(CDibDoc, CDocument)
	//{{AFX_MSG_MAP(CDibDoc)
	ON_COMMAND(ID_FILE_SAVE_COPY_AS, OnFileSaveCopyAs)
	ON_COMMAND(ID_OPTIONS_DROPLOGO, OnOptionsDropLogo)
	ON_COMMAND(ID_OPTIONS_NOSCALE, OnOptionsNoScale)
	ON_COMMAND(ID_OPTIONS_DROP10M, OnOptionsDrop10m)
	ON_COMMAND(ID_OPTIONS_DROP20M, OnOptionsDrop20m)
	ON_COMMAND(ID_OPTIONS_DROP50M, OnOptionsDrop50m)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_DROPLOGO, OnUpdateOptionsDropLogo)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_NOSCALE, OnUpdateOptionsNoScale)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_DROP10M, OnUpdateOptionsDrop10m)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_DROP20M, OnUpdateOptionsDrop20m)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_DROP50M, OnUpdateOptionsDrop50m)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDibDoc construction/destruction

CDibDoc::CDibDoc()
{
	m_hDIB = NULL;
	m_palDIB = NULL;
	m_sizeDoc = CSize(1,1);     // dummy value to make CScrollView happy
	m_MCUwidth = 8;
	m_MCUheight = 8;
	m_crop_xoffset = 0;
	m_crop_yoffset = 0;
	m_crop_width = 0;
	m_crop_height = 0;
	m_drop_logo = ((CDibLookApp *)AfxGetApp())->m_drop_logo_pre;
	m_drop_legend = ((CDibLookApp *)AfxGetApp())->m_drop_legend_pre;
}

CDibDoc::~CDibDoc()
{
	if (m_hDIB != NULL)
	{
		::GlobalFree((HGLOBAL) m_hDIB);
	}
	if (m_palDIB != NULL)
	{
		delete m_palDIB;
	}
}

BOOL CDibDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

void CDibDoc::InitDIBData()
{
	if (m_palDIB != NULL)
	{
		delete m_palDIB;
		m_palDIB = NULL;
	}
	if (m_hDIB == NULL)
	{
		return;
	}
	// Set up document size
	LPSTR lpDIB = (LPSTR) ::GlobalLock((HGLOBAL) m_hDIB);
	if (::DIBWidth(lpDIB) > INT_MAX ||::DIBHeight(lpDIB) > INT_MAX)
	{
		::GlobalUnlock((HGLOBAL) m_hDIB);
		::GlobalFree((HGLOBAL) m_hDIB);
		m_hDIB = NULL;
		CString strMsg;
		strMsg.LoadString(IDS_DIB_TOO_BIG);
		MessageBox(NULL, strMsg, NULL, MB_ICONINFORMATION | MB_OK);
		return;
	}
	m_sizeDoc = CSize((int) ::DIBWidth(lpDIB), (int) ::DIBHeight(lpDIB));
	::GlobalUnlock((HGLOBAL) m_hDIB);
	// Create copy of palette
	m_palDIB = new CPalette;
	if (m_palDIB == NULL)
	{
		// we must be really low on memory
		::GlobalFree((HGLOBAL) m_hDIB);
		m_hDIB = NULL;
		return;
	}
	if (::CreateDIBPalette(m_hDIB, m_palDIB) == NULL)
	{
		// DIB may not have a palette
		delete m_palDIB;
		m_palDIB = NULL;
		return;
	}
}


BOOL CDibDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
	DeleteContents();
	BeginWaitCursor();

	// replace calls to Serialize with ReadDIBFile function
	m_hDIB = (HDIB) Read_JPEG_File(lpszPathName, &m_MCUwidth, &m_MCUheight);

	InitDIBData();
	EndWaitCursor();

	if (m_hDIB == NULL)
	{
		// may not be DIB format
		CString strMsg;
		strMsg.LoadString(IDS_CANNOT_LOAD_DIB);
		MessageBox(NULL, strMsg, NULL, MB_ICONINFORMATION | MB_OK);
		return FALSE;
	}
	SetPathName(lpszPathName);
	SetModifiedFlag(FALSE);     // start off with unmodified
	return TRUE;
}


BOOL CDibDoc::OnSaveDocument(LPCTSTR lpszPathName)
{
	BOOL bSuccess;

	BeginWaitCursor();

	// replace calls to Serialize with SaveDIB function
	bSuccess = docrop(GetPathName(), lpszPathName,
					  m_crop_xoffset, m_crop_yoffset,
					  m_crop_width, m_crop_height,
					  "",
					  ((CDibLookApp *)AfxGetApp())->m_optimize_coding,
					  ((CDibLookApp *)AfxGetApp())->m_copyoption);
	if (bSuccess)
	{
	  if (m_drop_logo)
	  {
		((CDibLookApp *)AfxGetApp())->m_sampptr[0][0] = '0' + (char)(m_MCUwidth / 8);
		((CDibLookApp *)AfxGetApp())->m_sampptr[0][2] = '0' + (char)(m_MCUheight / 8);
		bSuccess = dodrop(lpszPathName, ((CDibLookApp *)AfxGetApp())->m_path[0],
						  lpszPathName,
						  1, 0, 2, 0,
						  "",
						  ((CDibLookApp *)AfxGetApp())->m_optimize_coding,
						  ((CDibLookApp *)AfxGetApp())->m_copyoption);
	  }
	  if (bSuccess)
	  {
		if (m_drop_legend)
		{
		  ((CDibLookApp *)AfxGetApp())->m_sampptr[m_drop_legend][0] = '0' + (char)(m_MCUwidth / 8);
		  ((CDibLookApp *)AfxGetApp())->m_sampptr[m_drop_legend][2] = '0' + (char)(m_MCUheight / 8);
		  bSuccess = dodrop(lpszPathName, ((CDibLookApp *)AfxGetApp())->m_path[m_drop_legend],
							lpszPathName,
							2, 0, 2, 0,
							"",
							((CDibLookApp *)AfxGetApp())->m_optimize_coding,
							((CDibLookApp *)AfxGetApp())->m_copyoption);
		}
	  }
	}

	EndWaitCursor();
	SetModifiedFlag(FALSE);     // back to unmodified

	if (!bSuccess)
	{
		// may be other-style DIB (load supported but not save)
		//  or other problem in SaveDIB
		CString strMsg;
		strMsg.LoadString(IDS_CANNOT_SAVE_DIB);
		MessageBox(NULL, strMsg, NULL, MB_ICONINFORMATION | MB_OK);
	}

	return bSuccess;
}

void CDibDoc::ReplaceHDIB(HDIB hDIB)
{
	if (m_hDIB != NULL)
	{
		::GlobalFree((HGLOBAL) m_hDIB);
	}
	m_hDIB = hDIB;
}

/////////////////////////////////////////////////////////////////////////////
// CDibDoc diagnostics

#ifdef _DEBUG
void CDibDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDibDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDibDoc commands

void CDibDoc::OnFileSaveCopyAs()
{
	DoSave(NULL, FALSE);
}

void CDibDoc::OnOptionsDropLogo()
{
  m_drop_logo ^= 1;
}

void CDibDoc::OnUpdateOptionsDropLogo(CCmdUI *pCmdUI)
{
  pCmdUI->SetCheck(m_drop_logo);
  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  if (myapp->m_logo_menu[0])
	pCmdUI->SetText(myapp->m_logo_menu);
}

void CDibDoc::OnOptionsNoScale()
{
  m_drop_legend = 0;
}

void CDibDoc::OnUpdateOptionsNoScale(CCmdUI *pCmdUI)
{
  pCmdUI->SetCheck(m_drop_legend == 0);
  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  if (myapp->m_legend_menu[0][0])
	pCmdUI->SetText(myapp->m_legend_menu[0]);
}

void CDibDoc::OnOptionsDrop10m()
{
  m_drop_legend = 1;
}

void CDibDoc::OnUpdateOptionsDrop10m(CCmdUI *pCmdUI)
{
  pCmdUI->SetCheck(m_drop_legend == 1);
  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  if (myapp->m_legend_menu[1][0])
	pCmdUI->SetText(myapp->m_legend_menu[1]);
}

void CDibDoc::OnOptionsDrop20m()
{
  m_drop_legend = 2;
}

void CDibDoc::OnUpdateOptionsDrop20m(CCmdUI *pCmdUI)
{
  pCmdUI->SetCheck(m_drop_legend == 2);
  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  if (myapp->m_legend_menu[2][0])
	pCmdUI->SetText(myapp->m_legend_menu[2]);
}

void CDibDoc::OnOptionsDrop50m()
{
  m_drop_legend = 3;
}

void CDibDoc::OnUpdateOptionsDrop50m(CCmdUI *pCmdUI)
{
  pCmdUI->SetCheck(m_drop_legend == 3);
  CDibLookApp *myapp = (CDibLookApp *) AfxGetApp();
  if (myapp->m_legend_menu[3][0])
	pCmdUI->SetText(myapp->m_legend_menu[3]);
}
